@@ -5,7 +5,7 @@
  * libradius.h	Structures and prototypes
  *		for the radius library.
  *
- * Version:	$Id: libradius.h,v 1.76.2.2.2.8 2006/05/16 18:26:06 aland Exp $
+ * Version:	$Id: libradius.h,v 1.76.2.2.2.10 2006/07/13 23:32:56 aland Exp $
  *
  */
 
@@ -65,12 +65,13 @@
 #define CHAP_VALUE_LENGTH       16
 #define MAX_STRING_LEN		254	/* RFC2138: string 0-253 octets */
 
+#  define VENDOR(x)		((x >> 16) & 0xffff)
+
 #ifdef _LIBRADIUS
 #  define AUTH_HDR_LEN		20
 #  define VENDORPEC_USR		429
 #define VENDORPEC_LUCENT	4846
 #define VENDORPEC_STARENT	8164
-#  define VENDOR(x)		((x >> 16) & 0xffff)
 #  define DEBUG			if (librad_debug) printf
 #  define debug_pair(vp)	do { if (librad_debug) { \
 					putchar('\t'); \
@@ -387,15 +388,19 @@ uint32_t lrad_hash_fold(uint32_t hash, int bits);
 
 typedef struct lrad_hash_table_t lrad_hash_table_t;
 typedef void (*lrad_hash_table_free_t)(void *);
+typedef uint32_t (*lrad_hash_table_hash_t)(const void *);
+typedef int (*lrad_hash_table_cmp_t)(const void *, const void *);
 typedef int (*lrad_hash_table_walk_t)(void * /* ctx */, void * /* data */);
 
-lrad_hash_table_t *lrad_hash_table_create(lrad_hash_table_free_t freeNode);
+lrad_hash_table_t *lrad_hash_table_create(lrad_hash_table_hash_t hashNode,
+					  lrad_hash_table_cmp_t cmpNode,
+					  lrad_hash_table_free_t freeNode);
 void		lrad_hash_table_free(lrad_hash_table_t *ht);
-int		lrad_hash_table_insert(lrad_hash_table_t *ht, uint32_t key, void *data);
-int		lrad_hash_table_delete(lrad_hash_table_t *ht, uint32_t key);
-void		*lrad_hash_table_yank(lrad_hash_table_t *ht, uint32_t key);
-int		lrad_hash_table_replace(lrad_hash_table_t *ht, uint32_t key, void *data);
-void		*lrad_hash_table_finddata(lrad_hash_table_t *ht, uint32_t key);
+int		lrad_hash_table_insert(lrad_hash_table_t *ht, void *data);
+int		lrad_hash_table_delete(lrad_hash_table_t *ht, const void *data);
+void		*lrad_hash_table_yank(lrad_hash_table_t *ht, const void *data);
+int		lrad_hash_table_replace(lrad_hash_table_t *ht, void *data);
+void		*lrad_hash_table_finddata(lrad_hash_table_t *ht, const void *data);
 int		lrad_hash_table_num_elements(lrad_hash_table_t *ht);
 int		lrad_hash_table_walk(lrad_hash_table_t *ht,
 				     lrad_hash_table_walk_t callback,
