@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: conffile.c,v 1.100.2.1 2005/10/03 17:18:59 nbk Exp $
+ * Version:	$Id: conffile.c,v 1.100.2.1.2.1 2006/04/19 16:21:50 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
 #include "modules.h"
 
 static const char rcsid[] =
-"$Id: conffile.c,v 1.100.2.1 2005/10/03 17:18:59 nbk Exp $";
+"$Id: conffile.c,v 1.100.2.1.2.1 2006/04/19 16:21:50 aland Exp $";
 
 #define xstrdup strdup
 
@@ -1038,6 +1038,10 @@ int cf_item_is_section(CONF_ITEM *item)
 {
 	return item->type == CONF_ITEM_SECTION;
 }
+int cf_item_is_pair(CONF_ITEM *item)
+{
+	return item->type == CONF_ITEM_PAIR;
+}
 
 
 #if 0
