@@ -1,7 +1,7 @@
 /*
  * proxy.c	Proxy stuff.
  *
- * Version:	$Id: proxy.c,v 1.77.2.2 2005/08/19 21:51:25 aland Exp $
+ * Version:	$Id: proxy.c,v 1.77.2.3.2.1 2005/12/08 13:24:32 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Chris Parker <cparker@starnetusa.com>
  */
 
-static const char rcsid[] = "$Id: proxy.c,v 1.77.2.2 2005/08/19 21:51:25 aland Exp $";
+static const char rcsid[] = "$Id: proxy.c,v 1.77.2.3.2.1 2005/12/08 13:24:32 nbk Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -53,6 +53,8 @@ static const char rcsid[] = "$Id: proxy.c,v 1.77.2.2 2005/08/19 21:51:25 aland E
 int proxy_receive(REQUEST *request)
 {
         int rcode;
+	int post_proxy_type = 0;
+	VALUE_PAIR *vp;
 
         /*
          *	Delete any reply we had accumulated until now.
@@ -63,7 +65,12 @@ int proxy_receive(REQUEST *request)
 	 *	Run the packet through the post-proxy stage,
 	 *	BEFORE playing games with the attributes.
 	 */
-        rcode = module_post_proxy(request);
+	vp = pairfind(request->config_items, PW_POST_PROXY_TYPE);
+	if (vp) {
+		DEBUG2("  Found Post-Proxy-Type %s", vp->strvalue);
+		post_proxy_type = vp->lvalue;
+	}
+	rcode = module_post_proxy(post_proxy_type, request);
 
         /*
          *	Delete the Proxy-State Attributes from the reply.
@@ -80,9 +87,8 @@ int proxy_receive(REQUEST *request)
         request->proxy_reply->vps = NULL;
 
         /*
-         *	Free any other configuration items and proxy pairs
+	 *	Free proxy request pairs.
          */
-        pairfree(&request->config_items);
         pairfree(&request->proxy->vps);
 
         return rcode;
@@ -229,6 +235,7 @@ static REALM *proxy_realm_ldb(REQUEST *request, const char *realm_name,
 int proxy_send(REQUEST *request)
 {
 	int rcode;
+	int pre_proxy_type = 0;
 	VALUE_PAIR *realmpair;
 	VALUE_PAIR *strippedname;
 	VALUE_PAIR *delaypair;
@@ -454,7 +461,12 @@ int proxy_send(REQUEST *request)
 	/*
 	 *  Do pre-proxying
 	 */
-	rcode = module_pre_proxy(request);
+	vp = pairfind(request->config_items, PW_PRE_PROXY_TYPE);
+	if (vp) {
+		DEBUG2("  Found Pre-Proxy-Type %s", vp->strvalue);
+		pre_proxy_type = vp->lvalue;
+	}
+	rcode = module_pre_proxy(pre_proxy_type, request);
 
 	/*
 	 *	Do NOT free request->proxy->vps, the pairs are needed
