@@ -1,7 +1,7 @@
 /*
  * radclient.c	General radius packet debug tool.
  *
- * Version:	$Id: radclient.c,v 1.72.2.1.2.4 2006/03/07 18:10:10 aland Exp $
+ * Version:	$Id: radclient.c,v 1.72.2.1.2.5 2006/05/16 18:26:08 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2000  Miquel van Smoorenburg <miquels@cistron.nl>
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
-static const char rcsid[] = "$Id: radclient.c,v 1.72.2.1.2.4 2006/03/07 18:10:10 aland Exp $";
+static const char rcsid[] = "$Id: radclient.c,v 1.72.2.1.2.5 2006/05/16 18:26:08 aland Exp $";
 
 #include "autoconf.h"
 
@@ -190,14 +190,14 @@ static radclient_t *radclient_init(const char *filename)
 		 */
 		radclient = malloc(sizeof(*radclient));
 		if (!radclient) {
-			perror("radclient: ");
+			perror("radclient: X");
 			return NULL; /* memory leak "start" */
 		}
 		memset(radclient, 0, sizeof(*radclient));
 
 		radclient->request = rad_alloc(1);
 		if (!radclient->request) {
-			librad_perror("radclient: ");
+			librad_perror("radclient: X");
 			radclient_free(radclient);
 			return NULL; /* memory leak "start" */
 		}
@@ -209,7 +209,7 @@ static radclient_t *radclient_init(const char *filename)
 		/*
 		 *	Read the VP's.
 		 */
-		radclient->request->vps = readvp2(fp, &filedone, "radclient:");
+		radclient->request->vps = readvp2(fp, &filedone, "radclient: X");
 		if (!radclient->request->vps) {
 			radclient_free(radclient);
 			return start; /* done: return the list */
@@ -640,6 +640,12 @@ static int recv_one_packet(int wait_time)
 	/*
 	 *	FIXME: Do stuff to process the reply.
 	 */
+	if (rad_verify(reply, radclient->request, secret) != 0) {
+		librad_perror("rad_verify");
+		totallost++;
+		goto packet_done; /* shared secret is incorrect */
+	}
+
 	if (rad_decode(reply, radclient->request, secret) != 0) {
 		librad_perror("rad_decode");
 		totallost++;
@@ -788,7 +794,7 @@ int main(int argc, char **argv)
 			timeout = atof(optarg);
 			break;
 		case 'v':
-			printf("radclient: $Id: radclient.c,v 1.72.2.1.2.4 2006/03/07 18:10:10 aland Exp $ built on " __DATE__ " at " __TIME__ "\n");
+			printf("radclient: $Id: radclient.c,v 1.72.2.1.2.5 2006/05/16 18:26:08 aland Exp $ built on " __DATE__ " at " __TIME__ "\n");
 			exit(0);
 			break;
 		case 'x':
