@@ -3,7 +3,7 @@
  *		Can also be installed as fingerd on the UNIX
  *		machine RADIUS runs on.
  *
- * Version:	$Id: radwho.c,v 1.44.2.3 2005/08/18 17:26:42 aland Exp $
+ * Version:	$Id: radwho.c,v 1.44.2.3.2.1 2005/12/14 18:32:02 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 static const char rcsid[] =
-"$Id: radwho.c,v 1.44.2.3 2005/08/18 17:26:42 aland Exp $";
+"$Id: radwho.c,v 1.44.2.3.2.1 2005/12/14 18:32:02 nbk Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -251,7 +251,7 @@ static const char *hostname(char *buf, size_t buflen, uint32_t ipaddr)
 /*
  *	Print usage message and exit.
  */
-static void usage(int status)
+static void NEVER_RETURNS usage(int status)
 {
 	FILE *output = status?stderr:stdout;
 
