@@ -2,7 +2,7 @@
  * request_list.c	Hide the handling of the REQUEST list from
  *			the main server.
  *
- * Version:	$Id: request_list.c,v 1.40.2.2 2004/06/01 14:35:55 aland Exp $
+ * Version:	$Id: request_list.c,v 1.40.2.2.2.1 2005/11/30 22:21:43 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  *
  * Copyright 2003-2004  The FreeRADIUS server project
  */
-static const char rcsid[] = "$Id: request_list.c,v 1.40.2.2 2004/06/01 14:35:55 aland Exp $";
+static const char rcsid[] = "$Id: request_list.c,v 1.40.2.2.2.1 2005/11/30 22:21:43 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -105,6 +105,8 @@ static rbtree_t		*proxy_id_tree;
  */
 static int		proxy_fds[32];
 
+static uint32_t		proxy_ipaddr;
+
 /*
  *	We can use 256 RADIUS Id's per dst ipaddr/port, per server
  *	socket.  So, to allocate them, we key off of dst ipaddr/port,
@@ -316,6 +318,7 @@ int rl_init(void)
 		     listener != NULL;
 		     listener = listener->next) {
 			if (listener->type == RAD_LISTEN_PROXY) {
+				proxy_ipaddr = listener->ipaddr;
 				proxy_fds[listener->fd & 0x1f] = listener->fd;
 				break;
 			}
@@ -791,6 +794,7 @@ int rl_add_proxy(REQUEST *request)
 	 */
 	entry->id[found] |= (1 << proxy);
 	request->proxy->id = found;
+	request->proxy->src_ipaddr = proxy_ipaddr;
 
 	rad_assert(proxy_fds[proxy] != -1);
 	request->proxy->sockfd = proxy_fds[proxy];
