@@ -2,7 +2,7 @@
  * xlat.c	Translate strings.  This is the first version of xlat
  * 		incorporated to RADIUS
  *
- * Version:	$Id: xlat.c,v 1.72.2.7.2.1 2005/12/08 12:47:56 nbk Exp $
+ * Version:	$Id: xlat.c,v 1.72.2.7.2.2 2006/07/13 16:40:54 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 static const char rcsid[] =
-"$Id: xlat.c,v 1.72.2.7.2.1 2005/12/08 12:47:56 nbk Exp $";
+"$Id: xlat.c,v 1.72.2.7.2.2 2006/07/13 16:40:54 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -142,7 +142,7 @@ static int xlat_packet(void *instance, REQUEST *request,
 	 */
 	da = dict_attrbyname(fmt);
 	if (!da) {
-		int index;
+		int count;
 		const char *p = strchr(fmt, '[');
 		char buffer[256];
 
@@ -151,7 +151,7 @@ static int xlat_packet(void *instance, REQUEST *request,
 
 		strNcpy(buffer, fmt, p - fmt + 1);
 
-		index = atoi(p + 1);
+		count = atoi(p + 1);
 
 		/*
 		 *	Check the format of the index before looking
@@ -175,8 +175,8 @@ static int xlat_packet(void *instance, REQUEST *request,
 		for (vp = pairfind(vps, da->attr);
 		     vp != NULL;
 		     vp = pairfind(vp->next, da->attr)) {
-			if (index == 0) break;
-			index--;
+			if (count == 0) break;
+			count--;
 		}
 
 		/*
@@ -533,7 +533,7 @@ static void decode_attribute(const char **from, char **to, int freespace,
 	 * useless if we found what we need
 	 */
 	if (found) {
-		while((*p != '\0') && (openbraces > 0)) {
+		while((*p != '\0') && (openbraces > *open)) {
 			/*
 			 *	Handle escapes outside of the loop.
 			 */
