@@ -1,7 +1,7 @@
 /*
  * rlm_chap.c
  *
- * Version:  $Id: rlm_chap.c,v 1.4 2002/08/24 16:54:56 aland Exp $
+ * Version:  $Id: rlm_chap.c,v 1.7 2004/02/26 19:04:27 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -36,27 +36,31 @@
 #include "modules.h"
 #include "conffile.h"
 
-static const char rcsid[] = "$Id: rlm_chap.c,v 1.4 2002/08/24 16:54:56 aland Exp $";
+static const char rcsid[] = "$Id: rlm_chap.c,v 1.7 2004/02/26 19:04:27 aland Exp $";
 
 static int chap_authorize(void *instance, REQUEST *request)
 {
-	
+
 	/* quiet the compiler */
 	instance = instance;
 	request = request;
 
-	if (!request->password || request->password->attribute != PW_CHAP_PASSWORD){
-		DEBUG("rlm_chap: Could not find proper Chap-Password attribute in request");
+	if (!request->password ||
+	    request->password->attribute != PW_CHAP_PASSWORD) {
 		return RLM_MODULE_NOOP;
 	}
-	if (pairfind(request->config_items, PW_AUTHTYPE) == NULL){
-		DEBUG("rlm_chap: Adding Auth-Type = CHAP");
-		pairadd(&request->config_items, pairmake("Auth-Type", "CHAP", T_OP_EQ));
+
+	if (pairfind(request->config_items, PW_AUTHTYPE) != NULL) {
+		DEBUG2("  rlm_chap: WARNING: Auth-Type already set.  Not setting to CHAP");
+		return RLM_MODULE_NOOP;
 	}
 
+	DEBUG("  rlm_chap: Setting 'Auth-Type := CHAP'");
+	pairadd(&request->config_items,
+		pairmake("Auth-Type", "CHAP", T_OP_EQ));
 	return RLM_MODULE_OK;
 }
-	
+
 
 /*
  *	Find the named user in this modules database.  Create the set
@@ -75,12 +79,12 @@ static int chap_authenticate(void *instance, REQUEST *request)
 	instance = instance;
 	request = request;
 
-	if(!request->username){
+	if (!request->username) {
 		radlog(L_AUTH, "rlm_chap: Attribute \"User-Name\" is required for authentication.\n");
 		return RLM_MODULE_INVALID;
 	}
 
-	if (!request->password){
+	if (!request->password) {
 		radlog(L_AUTH, "rlm_chap: Attribute \"CHAP-Password\" is required for authentication.");
 		return RLM_MODULE_INVALID;
 	}
@@ -95,31 +99,34 @@ static int chap_authenticate(void *instance, REQUEST *request)
 		return RLM_MODULE_INVALID;
 	}
 
-	DEBUG("rlm_chap: login attempt by \"%s\" with CHAP password %s", 
-		request->username->strvalue, request->password->strvalue);
+	/*
+	 *	Don't print out the CHAP password here.  It's binary crap.
+	 */
+	DEBUG("  rlm_chap: login attempt by \"%s\" with CHAP password",
+		request->username->strvalue);
 
 	if ((passwd_item = pairfind(request->config_items, PW_PASSWORD)) == NULL){
-		DEBUG("rlm_chap: Could not find clear text password for user %s",request->username->strvalue);
+		DEBUG("  rlm_chap: Could not find clear text password for user %s",request->username->strvalue);
 		snprintf(module_fmsg,sizeof(module_fmsg),"rlm_chap: Clear text password not available");
 		module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
 		pairadd(&request->packet->vps, module_fmsg_vp);
 		return RLM_MODULE_INVALID;
 	}
 
-	DEBUG("rlm_chap: Using clear text password %s for user %s authentication.",
+	DEBUG("  rlm_chap: Using clear text password %s for user %s authentication.",
 	      passwd_item->strvalue, request->username->strvalue);
-	
+
 	rad_chap_encode(request->packet,pass_str,request->password->strvalue[0],passwd_item);
-	
+
 	if (memcmp(pass_str+1,request->password->strvalue+1,CHAP_VALUE_LENGTH) != 0){
-		DEBUG("rlm_chap: Pasword check failed");
+		DEBUG("  rlm_chap: Pasword check failed");
 		snprintf(module_fmsg,sizeof(module_fmsg),"rlm_chap: Wrong user password");
 		module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
 		pairadd(&request->packet->vps, module_fmsg_vp);
 		return RLM_MODULE_REJECT;
 	}
 
-	DEBUG("rlm_chap: chap user %s authenticated succesfully",request->username->strvalue);
+	DEBUG("  rlm_chap: chap user %s authenticated succesfully",request->username->strvalue);
 
 	return RLM_MODULE_OK;
 }
@@ -134,7 +141,7 @@ static int chap_authenticate(void *instance, REQUEST *request)
  *	is single-threaded.
  */
 module_t rlm_chap = {
-	"CHAP",	
+	"CHAP",
 	0,				/* type */
 	NULL,				/* initialization */
 	NULL,				/* instantiation */
