@@ -1,7 +1,7 @@
 /*
  * eap.h    Header file containing the interfaces for all EAP types.
  *
- * Version:     $Id: eap.h,v 1.27 2004/03/08 21:51:30 aland Exp $
+ * Version:     $Id: eap.h,v 1.27.4.1 2006/02/06 16:23:50 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -100,8 +100,7 @@ typedef enum operation_t {
  */
 #define EAP_STATE_LEN (AUTH_VECTOR_LEN)
 typedef struct _eap_handler {
-	struct _eap_handler *next;
-
+	struct _eap_handler *prev, *next;
 	uint8_t		state[EAP_STATE_LEN];
 	uint32_t	src_ipaddr;
 	unsigned int	eap_id;
