@@ -1,7 +1,7 @@
 /*
  * eap_tls.c
  *
- * Version:     $Id: eap_tls.c,v 1.1.2.1 2006/02/07 18:58:06 aland Exp $
+ * Version:     $Id: eap_tls.c,v 1.1.2.2 2006/05/19 14:19:14 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -37,6 +37,8 @@
  *
  */
 
+#include "autoconf.h"
+#include <assert.h>
 #include "eap_tls.h"
 
 /*
@@ -458,7 +460,7 @@ static EAPTLS_PACKET *eaptls_extract(EAP_DS *eap_ds, eaptls_status_t status)
 	 *	MUST have TLS type octet, followed by flags, followed
 	 *	by data.
 	 */
-	rad_assert(eap_ds->response->length > 2);
+	assert(eap_ds->response->length > 2);
 
 	tlspacket = eaptls_alloc();
 	if (tlspacket == NULL) return NULL;
