@@ -1,7 +1,7 @@
 /*
  * radeapclient.c	EAP specific radius packet debug tool.
  *
- * Version:	$Id: radeapclient.c,v 1.7.4.2 2006/02/06 16:23:51 nbk Exp $
+ * Version:	$Id: radeapclient.c,v 1.7.4.5 2006/05/19 14:22:23 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  * Copyright 2000  Miquel van Smoorenburg <miquels@cistron.nl>
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
-static const char rcsid[] = "$Id: radeapclient.c,v 1.7.4.2 2006/02/06 16:23:51 nbk Exp $";
+static const char rcsid[] = "$Id: radeapclient.c,v 1.7.4.5 2006/05/19 14:22:23 nbk Exp $";
 
 #include "autoconf.h"
 
@@ -116,6 +116,19 @@ int radlog(int lvl, const char *msg, ...)
 	return r;
 }
 
+int log_debug(const char *msg, ...)
+{
+	va_list ap;
+	int r;
+
+	va_start(ap, msg);
+	r = vfprintf(stderr, msg, ap);
+	va_end(ap);
+	fputc('\n', stderr);
+
+	return r;
+}
+
 static int getport(const char *name)
 {
 	struct	servent		*svp;
@@ -187,6 +200,11 @@ static int send_packet(RADIUS_PACKET *req, RADIUS_PACKET **rep)
 	 *	Hmm... we should really be using eapol_test, which does
 	 *	a lot more than radeapclient.
 	 */
+	if (rad_verify(*rep, req, secret) != 0) {
+		librad_perror("rad_verify");
+		exit(1);
+	}
+
 	if (rad_decode(*rep, req, secret) != 0) {
 		librad_perror("rad_decode");
 		exit(1);
@@ -928,7 +946,9 @@ int main(int argc, char **argv)
 			break;
 
 		case 'X':
-		  sha1_data_problems = 1;
+#if 0
+		  sha1_data_problems = 1; /* for debugging only */
+#endif
 		  break;
 
 
@@ -955,7 +975,7 @@ int main(int argc, char **argv)
 			timeout = atof(optarg);
 			break;
 		case 'v':
-			printf("radclient: $Id: radeapclient.c,v 1.7.4.2 2006/02/06 16:23:51 nbk Exp $ built on " __DATE__ " at " __TIME__ "\n");
+			printf("radclient: $Id: radeapclient.c,v 1.7.4.5 2006/05/19 14:22:23 nbk Exp $ built on " __DATE__ " at " __TIME__ "\n");
 			exit(0);
 			break;
                case 'S':
