@@ -1,7 +1,7 @@
 /*
  * rlm_eap_tls.c  contains the interfaces that are called from eap
  *
- * Version:     $Id: rlm_eap_tls.c,v 1.21.4.8 2006/04/28 18:20:42 aland Exp $
+ * Version:     $Id: rlm_eap_tls.c,v 1.21.4.10 2006/08/22 16:20:41 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -449,17 +449,20 @@ static int eaptls_detach(void *arg)
 	conf = inst->conf;
 
 	if (conf) {
-		if (conf->dh_file) free(conf->dh_file);
-		conf->dh_file = NULL;
-		if (conf->certificate_file) free(conf->certificate_file);
-		conf->certificate_file = NULL;
-		if (conf->private_key_file) free(conf->private_key_file);
-		conf->private_key_file = NULL;
-		if (conf->private_key_password) free(conf->private_key_password);
-		conf->private_key_password = NULL;
-		if (conf->random_file) free(conf->random_file);
-		conf->random_file = NULL;
+		free(conf->dh_file);
+		free(conf->ca_path);
+		free(conf->certificate_file);
+		free(conf->private_key_file);
+		free(conf->private_key_password);
+		free(conf->ca_file);
+		free(conf->random_file);
 
+		free(conf->check_cert_cn);
+		free(conf->check_cert_cn);
+		free(conf->cipher_list);
+		free(conf->check_cert_issuer);
+
+		memset(conf, 0, sizeof(*conf));
 		free(inst->conf);
 		inst->conf = NULL;
 	}
@@ -613,13 +616,33 @@ static int eaptls_initiate(void *type_arg, EAP_HANDLER *handler)
 	ssn->length_flag = inst->conf->include_length;
 
 	/*
-	 *	We set a default fragment size, unless the Framed-MTU
-	 *	tells us it's too big.
+	 *	We use default fragment size, unless the Framed-MTU
+	 *	tells us it's too big.  Note that we do NOT account
+	 *	for the EAP-TLS headers if conf->fragment_size is
+	 *	large, because that config item looks to be confusing.
+	 *
+	 *	i.e. it should REALLY be called MTU, and the code here
+	 *	should figure out what that means for TLS fragment size.
+	 *	asking the administrator to know the internal details
+	 *	of EAP-TLS in order to calculate fragment sizes is
+	 *	just too much.
 	 */
 	ssn->offset = inst->conf->fragment_size;
 	vp = pairfind(handler->request->packet->vps, PW_FRAMED_MTU);
-	if (vp && ((vp->lvalue - 4) < ssn->offset)) {
-		ssn->offset = vp->lvalue - 4;
+	if (vp && ((vp->lvalue - 14) < ssn->offset)) {
+		/*
+		 *	Discount the Framed-MTU by:
+		 *	 4 : EAPOL header
+		 *	 4 : EAP header (code + id + length)
+		 *	 1 : EAP type == EAP-TLS
+		 *	 1 : EAP-TLS Flags
+		 *	 4 : EAP-TLS Message length
+		 *	    (even if conf->include_length == 0,
+		 *	     just to be lazy).
+		 *	---
+		 *	14
+		 */
+		ssn->offset = vp->lvalue - 14;
 	}
 
 	handler->opaque = ((void *)ssn);
