@@ -1,7 +1,7 @@
 /*
  * eap_ttls.h
  *
- * Version:     $Id: eap_ttls.h,v 1.5 2004/02/26 19:04:32 aland Exp $
+ * Version:     $Id: eap_ttls.h,v 1.5.4.1 2006/02/06 16:23:57 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #ifndef _EAP_TTLS_H
 #define _EAP_TTLS_H
 
-#include "rlm_eap_tls.h"
+#include "eap_tls.h"
 
 typedef struct ttls_tunnel_t {
 	VALUE_PAIR	*username;
