@@ -1,7 +1,7 @@
 /*
  * rlm_eap_ttls.c  contains the interfaces that are called from eap
  *
- * Version:     $Id: ttls.c,v 1.17.2.2.2.1 2006/02/06 16:23:57 nbk Exp $
+ * Version:     $Id: ttls.c,v 1.17.2.2.2.2 2006/07/17 19:07:37 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -46,6 +46,11 @@ static int diameter_verify(const uint8_t *data, unsigned int data_len)
 	unsigned int data_left = data_len;
 
 	while (data_left > 0) {
+		if (data_len < 12) {
+			DEBUG2("  rlm_eap_ttls:  Diameter attribute is too small to contain a Diameter header");
+			return 0;
+		}
+
 		rad_assert(data_left <= data_len);
 		memcpy(&attr, data, sizeof(attr));
 		data += 4;
