@@ -1,7 +1,7 @@
 /*
  * rlm_expr.c
  *
- * Version:	$Id: rlm_expr.c,v 1.7 2004/02/26 19:04:32 aland Exp $
+ * Version:	$Id: rlm_expr.c,v 1.7.2.1 2005/08/24 14:37:52 nbk Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 #include "modules.h"
 #include "conffile.h"
 
-static const char rcsid[] = "$Id: rlm_expr.c,v 1.7 2004/02/26 19:04:32 aland Exp $";
+static const char rcsid[] = "$Id: rlm_expr.c,v 1.7.2.1 2005/08/24 14:37:52 nbk Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -74,7 +74,8 @@ static expr_map_t map[] =
 static int get_number(REQUEST *request, const char **string, int *answer)
 {
 	int		i, found;
-	uint32_t       	result, x;
+	uint32_t	result;
+	int		x;
 	const char	*p;
 	expr_token_t	this;
 
