@@ -1,7 +1,7 @@
 /*
  * rlm_mschap.c
  *
- * Version:	$Id: rlm_mschap.c,v 1.59.2.3 2005/10/19 16:49:46 mgriego Exp $
+ * Version:	$Id: rlm_mschap.c,v 1.59.2.3.2.2 2006/04/13 16:28:30 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -60,7 +60,7 @@
 
 #include	"smbdes.h"
 
-static const char rcsid[] = "$Id: rlm_mschap.c,v 1.59.2.3 2005/10/19 16:49:46 mgriego Exp $";
+static const char rcsid[] = "$Id: rlm_mschap.c,v 1.59.2.3.2.2 2006/04/13 16:28:30 aland Exp $";
 
 static const char *letters = "0123456789ABCDEF";
 
@@ -288,8 +288,8 @@ typedef struct rlm_mschap_t {
         int with_ntdomain_hack;	/* this should be in another module */
 	char *passwd_file;
 	char *xlat_name;
-	char *auth_type;	/* I don't think this is needed... */
 	char *ntlm_auth;
+	char *auth_type;
 } rlm_mschap_t;
 
 
@@ -600,8 +600,6 @@ static CONF_PARSER module_config[] = {
 	  offsetof(rlm_mschap_t,with_ntdomain_hack), NULL, "no" },
 	{ "passwd",   PW_TYPE_STRING_PTR,
 	  offsetof(rlm_mschap_t, passwd_file), NULL,  NULL },
-	{ "authtype",   PW_TYPE_STRING_PTR,
-	  offsetof(rlm_mschap_t, auth_type), NULL,  NULL },
 	{ "ntlm_auth",   PW_TYPE_STRING_PTR,
 	  offsetof(rlm_mschap_t, ntlm_auth), NULL,  NULL },
 
@@ -614,9 +612,8 @@ static CONF_PARSER module_config[] = {
  */
 static int mschap_detach(void *instance){
 #define inst ((rlm_mschap_t *)instance)
-	if (inst->passwd_file) free(inst->passwd_file);
-	if (inst->auth_type) free(inst->auth_type);
-	if (inst->ntlm_auth) free(inst->ntlm_auth);
+	free(inst->passwd_file);
+	free(inst->ntlm_auth);
 	if (inst->xlat_name) {
 		xlat_unregister(inst->xlat_name, mschap_xlat);
 		free(inst->xlat_name);
@@ -669,6 +666,13 @@ static int mschap_instantiate(CONF_SECTION *conf, void **instance)
 		xlat_register(xlat_name, mschap_xlat, inst);
 	}
 
+	/*
+	 *	For backwards compatibility
+	 */
+	if (!dict_valbyname(PW_AUTH_TYPE, inst->xlat_name)) {
+		inst->auth_type = "MS-CHAP";
+	}
+
 	return 0;
 }
 
@@ -958,7 +962,6 @@ static int mschap_authorize(void * instance, REQUEST *request)
 	VALUE_PAIR *challenge = NULL;
 	VALUE_PAIR *response = NULL;
 	VALUE_PAIR *vp;
-	const char *authtype_name = "MS-CHAP";
 
 	challenge = pairfind(request->packet->vps, PW_MSCHAP_CHALLENGE);
 	if (!challenge) {
@@ -977,24 +980,17 @@ static int mschap_authorize(void * instance, REQUEST *request)
 		return RLM_MODULE_NOOP;
 	}
 
-	/*
-	 *	Choose MS-CHAP, or whatever else they told us to use.
-	 */
-	if (inst->auth_type) {
-		authtype_name = inst->auth_type;
-	}
-
-	DEBUG2("  rlm_mschap: Found MS-CHAP attributes.  Setting 'Auth-Type  = %s'", authtype_name);
+	DEBUG2("  rlm_mschap: Found MS-CHAP attributes.  Setting 'Auth-Type  = %s'", inst->xlat_name);
 
 	/*
 	 *	Set Auth-Type to MS-CHAP.  The authentication code
 	 *	will take care of turning clear-text passwords into
 	 *	NT/LM passwords.
 	 */
-	pairdelete(&request->config_items, PW_AUTHTYPE);
-	vp = pairmake("Auth-Type", authtype_name, T_OP_EQ);
-	rad_assert(vp != NULL);
-	pairadd(&request->config_items, vp);
+	vp = pairmake("Auth-Type", inst->auth_type, T_OP_EQ);
+	if (!vp) return RLM_MODULE_FAIL;
+	pairmove(&request->config_items, &vp);
+	pairfree(&vp);		/* may be NULL */
 
 	return RLM_MODULE_OK;
 #undef inst
