@@ -1,7 +1,7 @@
 /*
  * rlm_pap.c
  *
- * Version:  $Id: rlm_pap.c,v 1.15.2.2 2005/08/24 14:37:52 nbk Exp $
+ * Version:  $Id: rlm_pap.c,v 1.15.2.2.2.1 2006/08/22 20:19:22 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -47,7 +47,7 @@
 	free((char *)inst->scheme); \
 	free(inst)
 
-static const char rcsid[] = "$Id: rlm_pap.c,v 1.15.2.2 2005/08/24 14:37:52 nbk Exp $";
+static const char rcsid[] = "$Id: rlm_pap.c,v 1.15.2.2.2.1 2006/08/22 20:19:22 aland Exp $";
 
 /*
  *      Define a structure for our module configuration.
@@ -193,7 +193,7 @@ static int pap_authenticate(void *instance, REQUEST *request)
 	if ((((passwd_item = pairfind(request->config_items, PW_PASSWORD)) == NULL) &&
 		((passwd_item = pairfind(request->config_items, PW_CRYPT_PASSWORD)) == NULL)) ||
 	    (passwd_item->length == 0) || (passwd_item->strvalue[0] == 0)) {
-		DEBUG("rlm_pap: No password (or empty password) to check against for for user %s",request->username->strvalue);
+		DEBUG("rlm_pap: No password (or empty password) to check against for user %s",request->username->strvalue);
 		snprintf(module_fmsg,sizeof(module_fmsg),"rlm_pap: User password not available");
 		module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
 		pairadd(&request->packet->vps, module_fmsg_vp);
