@@ -1,7 +1,7 @@
 /*
  * rlm_perl.c
  *
- * Version:    $Id: rlm_perl.c,v 1.13.4.4 2006/03/17 19:49:55 aland Exp $
+ * Version:    $Id: rlm_perl.c,v 1.13.4.7 2006/04/27 17:35:44 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@
 extern char **environ;
 #endif
 
-static const char rcsid[] = "$Id: rlm_perl.c,v 1.13.4.4 2006/03/17 19:49:55 aland Exp $";
+static const char rcsid[] = "$Id: rlm_perl.c,v 1.13.4.7 2006/04/27 17:35:44 aland Exp $";
 
 #ifdef USE_ITHREADS
 
@@ -592,17 +592,7 @@ static int init_pool (CONF_SECTION *conf, PERL_INST *inst) {
  */
 static int perl_init(void)
 {
-#ifdef USE_ITHREADS
-	if ((interp = perl_alloc()) == NULL) {
-		radlog(L_DBG, "rlm_perl: No memory for allocating new perl !");
-		return -1;
-	}
-
-	perl_construct(interp);
-	PL_perl_destruct_level = 2;
-#endif
 	return 0;
-
 }
 
 static void xs_init(pTHX)
@@ -653,6 +643,14 @@ static int perl_xlat(void *instance, REQUEST *request, char *fmt, char * out,
 	char		params[1024], *ptr, *tmp;
 	int		count, ret=0;
 	STRLEN		n_a;
+
+	/*
+	 * Do an xlat on the provided string (nice recursive operation).
+	*/
+	if (!radius_xlat(params, sizeof(params), fmt, request, func)) {
+		radlog(L_ERR, "rlm_perl: xlat failed.");
+		return 0;
+	}
 #ifndef USE_ITHREADS
 	perl = inst->perl;
 #endif
@@ -675,13 +673,6 @@ static int perl_xlat(void *instance, REQUEST *request, char *fmt, char * out,
 	dSP;
 	ENTER;SAVETMPS;
 
-	/*
-	 * Do an xlat on the provided string (nice recursive operation).
-	*/
-	if (!radius_xlat(params, sizeof(params), fmt, request, func)) {
-		radlog(L_ERR, "rlm_perl: xlat failed.");
-		return 0;
-	}
 	ptr = strtok(params, " ");
 
 	PUSHMARK(SP);
@@ -699,22 +690,20 @@ static int perl_xlat(void *instance, REQUEST *request, char *fmt, char * out,
 	if (SvTRUE(ERRSV)) {
 		radlog(L_ERR, "rlm_perl: perl_xlat exit %s\n",
 		       SvPV(ERRSV,n_a));
-		return 0;
-	}
-
-	if (count > 0) {
+		POPs ;
+	} else if (count > 0) {
 		tmp = POPp;
 		ret = strlen(tmp);
 		strncpy(out,tmp,ret);
 
 		radlog(L_DBG,"rlm_perl: Len is %d , out is %s freespace is %d",
 		       ret, out,freespace);
-
-		PUTBACK ;
-		FREETMPS ;
-		LEAVE ;
-
 	}
+
+	PUTBACK ;
+	FREETMPS ;
+	LEAVE ;
+
 	}
 #ifdef USE_ITHREADS
 	pool_release(handle, instance);
@@ -779,6 +768,15 @@ static int perl_instantiate(CONF_SECTION *conf, void **instance)
 
 #ifdef USE_ITHREADS
 	inst->perl = interp;
+	
+	if ((inst->perl = perl_alloc()) == NULL) {
+		radlog(L_DBG, "rlm_perl: No memory for allocating new perl !");
+		return (-1);
+	}
+
+	perl_construct(inst->perl);
+	PL_perl_destruct_level = 2;
+
 	{
 	dTHXa(inst->perl);
 	}
@@ -895,13 +893,13 @@ static void perl_store_vps(VALUE_PAIR *vp, HV *rad_hv)
  *     Value Pair Format
  *
  */
-static int pairadd_sv(VALUE_PAIR **vp, char *key, SV *sv) {
+static int pairadd_sv(VALUE_PAIR **vp, char *key, SV *sv, int operator) {
        char            *val;
        VALUE_PAIR      *vpp;
 
        if (SvOK(sv)) {
                val = SvPV_nolen(sv);
-               vpp = pairmake(key, val, T_OP_EQ);
+               vpp = pairmake(key, val, operator);
                if (vpp != NULL) {
                        pairadd(vp, vpp);
                        radlog(L_DBG,
@@ -935,9 +933,9 @@ static int get_hv_content(HV *my_hv, VALUE_PAIR **vp)
                        len = av_len(av);
                        for (j = 0; j <= len; j++) {
                                av_sv = av_fetch(av, j, 0);
-                               ret = pairadd_sv(vp, key, *av_sv) + ret;
+                               ret = pairadd_sv(vp, key, *av_sv, T_OP_ADD) + ret;
                        }
-               } else ret = pairadd_sv(vp, key, res_sv) + ret;
+               } else ret = pairadd_sv(vp, key, res_sv, T_OP_EQ) + ret;
         }
 
         return ret;
@@ -1032,22 +1030,25 @@ static int rlmperl_call(void *instance, REQUEST *request, char *function_name)
 
 	SPAGAIN;
 
+	if (SvTRUE(ERRSV)) {
+		radlog(L_ERR, "rlm_perl: perl_embed:: module = %s , func = %s exit status= %s\n",
+		       inst->module,
+		       function_name, SvPV(ERRSV,n_a));
+		POPs;
+	}
+
 	if (count == 1) {
 		exitstatus = POPi;
 		if (exitstatus >= 100 || exitstatus < 0) {
 			exitstatus = RLM_MODULE_FAIL;
 		}
 	}
+	
 
 	PUTBACK;
 	FREETMPS;
 	LEAVE;
 
-	if (SvTRUE(ERRSV)) {
-		radlog(L_ERR, "rlm_perl: perl_embed:: module = %s , func = %s exit status= %s\n",
-		       inst->module,
-		       function_name, SvPV(ERRSV,n_a));
-	}
 
 	if ((get_hv_content(rad_reply_hv, &vp)) > 0 ) {
 		pairmove(&request->reply->vps, &vp);
