@@ -3,7 +3,7 @@
  *		Contains the functions for the "huntgroups" and "hints"
  *		files.
  *
- * Version:     $Id: rlm_preprocess.c,v 1.52.2.1.2.1 2006/05/05 17:31:53 aland Exp $
+ * Version:     $Id: rlm_preprocess.c,v 1.52.2.1.2.3 2006/07/14 16:54:11 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: rlm_preprocess.c,v 1.52.2.1.2.1 2006/05/05 17:31:53 aland Exp $";
+static const char rcsid[] = "$Id: rlm_preprocess.c,v 1.52.2.1.2.3 2006/07/14 16:54:11 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -48,6 +48,7 @@ typedef struct rlm_preprocess_t {
 	int		with_ntdomain_hack;
 	int		with_specialix_jetstream_hack;
 	int		with_cisco_vsa_hack;
+	int		with_alvarion_vsa_hack;
 } rlm_preprocess_t;
 
 static CONF_PARSER module_config[] = {
@@ -69,6 +70,8 @@ static CONF_PARSER module_config[] = {
 	  "no" },
 	{ "with_cisco_vsa_hack",        PW_TYPE_BOOLEAN,
 	  offsetof(rlm_preprocess_t,with_cisco_vsa_hack), NULL, "no" },
+	{ "with_alvarion_vsa_hack",        PW_TYPE_BOOLEAN,
+	  offsetof(rlm_preprocess_t,with_alvarion_vsa_hack), NULL, "no" },
 
 	{ NULL, -1, 0, NULL, NULL }
 };
@@ -115,7 +118,7 @@ static void cisco_vsa_hack(VALUE_PAIR *vp)
 	char		newattr[MAX_STRING_LEN];
 
 	for ( ; vp != NULL; vp = vp->next) {
-		vendorcode = (vp->attribute >> 16); /* HACK! */
+		vendorcode = VENDOR(vp->attribute);
 		if (!((vendorcode == 9) || (vendorcode == 6618))) continue; /* not a Cisco or Quintum VSA, continue */
 
 		if (vp->type != PW_TYPE_STRING) continue;
@@ -170,6 +173,26 @@ static void cisco_vsa_hack(VALUE_PAIR *vp)
 	}
 }
 
+
+/*
+ *	Don't even ask what this is doing...
+ */
+static void alvarion_vsa_hack(VALUE_PAIR *vp)
+{
+	int		vendorcode;
+	int		number = 1;
+
+	for ( ; vp != NULL; vp = vp->next) {
+		vendorcode = VENDOR(vp->attribute);
+		if (vendorcode != 12394) continue;
+		if (vp->type != PW_TYPE_STRING) continue;
+
+		vp->attribute = number | (12394 << 16);
+		snprintf(vp->name, sizeof(vp->name),
+			 "Breezecom-Attr%d", number++);
+	}
+}
+
 /*
  *	Mangle username if needed, IN PLACE.
  */
@@ -515,6 +538,14 @@ static int preprocess_authorize(void *instance, REQUEST *request)
 		cisco_vsa_hack(request->packet->vps);
 	}
 
+	if (data->with_alvarion_vsa_hack) {
+	 	/*
+		 *	We need to run this hack because the Alvarion
+		 *	people are crazy.
+		 */
+		alvarion_vsa_hack(request->packet->vps);
+	}
+
 	/*
 	 *	Note that we add the Request-Src-IP-Address to the request
 	 *	structure BEFORE checking huntgroup access.  This allows
@@ -578,6 +609,14 @@ static int preprocess_preaccounting(void *instance, REQUEST *request)
 		cisco_vsa_hack(request->packet->vps);
 	}
 
+	if (data->with_alvarion_vsa_hack) {
+	 	/*
+		 *	We need to run this hack because the Alvarion
+		 *	people are crazy.
+		 */
+		alvarion_vsa_hack(request->packet->vps);
+	}
+
 	/*
 	 *  Ensure that we log the NAS IP Address in the packet.
 	 */
